# Authors: Gael Varoquaux <gael.varoquaux@normalesup.org>
#          Justin Vincent
#          Lars Buitinck
# License: BSD 3 clause

import pickle

import numpy as np
import pytest

from sklearn.utils.testing import assert_equal
from sklearn.utils.testing import assert_array_equal
from sklearn.utils.testing import assert_allclose

from sklearn.utils.fixes import divide
from sklearn.utils.fixes import MaskedArray
from sklearn.utils.fixes import nanmedian
from sklearn.utils.fixes import nanpercentile


def test_divide():
    assert_equal(divide(.6, 1), .600000000000)


def test_masked_array_obj_dtype_pickleable():
    marr = MaskedArray([1, None, 'a'], dtype=object)

    for mask in (True, False, [0, 1, 0]):
        marr.mask = mask
        marr_pickled = pickle.loads(pickle.dumps(marr))
        assert_array_equal(marr.data, marr_pickled.data)
        assert_array_equal(marr.mask, marr_pickled.mask)


@pytest.mark.parametrize(
    "axis, expected_median",
    [(None, 4.0),
     (0, np.array([1., 3.5, 3.5, 4., 7., np.nan])),
     (1, np.array([1., 6.]))]
)
def test_nanmedian(axis, expected_median):
    X = np.array([[1, 1, 1, 2, np.nan, np.nan],
                  [np.nan, 6, 6, 6, 7, np.nan]])
    median = nanmedian(X, axis=axis)
    if axis is None:
        assert median == pytest.approx(expected_median)
    else:
        assert_allclose(median, expected_median)


@pytest.mark.parametrize(
    "a, q, expected_percentile",
    [(np.array([1, 2, 3, np.nan]), [0, 50, 100], np.array([1., 2., 3.])),
     (np.array([1, 2, 3, np.nan]), 50, 2.),
     (np.array([np.nan, np.nan]), [0, 50], np.array([np.nan, np.nan]))]
)
def test_nanpercentile(a, q, expected_percentile):
    percentile = nanpercentile(a, q)
    assert_allclose(percentile, expected_percentile)



"""
The :mod:`sklearn.cluster` module gathers popular unsupervised clustering
algorithms.
"""

from .spectral import spectral_clustering, SpectralClustering
from .mean_shift_ import (mean_shift, MeanShift,
                          estimate_bandwidth, get_bin_seeds)
from .affinity_propagation_ import affinity_propagation, AffinityPropagation
from .hierarchical import (ward_tree, AgglomerativeClustering, linkage_tree,
                           FeatureAgglomeration)
from .k_means_ import k_means, KMeans, MiniBatchKMeans
from .dbscan_ import dbscan, DBSCAN
from .bicluster import SpectralBiclustering, SpectralCoclustering
from .birch import Birch

__all__ = ['AffinityPropagation',
           'AgglomerativeClustering',
           'Birch',
           'DBSCAN',
           'KMeans',
           'FeatureAgglomeration',
           'MeanShift',
           'MiniBatchKMeans',
           'SpectralClustering',
           'affinity_propagation',
           'dbscan',
           'estimate_bandwidth',
           'get_bin_seeds',
           'k_means',
           'linkage_tree',
           'mean_shift',
           'spectral_clustering',
           'ward_tree',
           'SpectralBiclustering',
           'SpectralCoclustering']



import os
import pytest
from sklearn import externals
from sklearn.externals import joblib as joblib_vendored
from sklearn.utils import Parallel, delayed, Memory, parallel_backend

if os.environ.get('SKLEARN_SITE_JOBLIB', False):
    import joblib as joblib_site
else:
    joblib_site = None


def test_old_pickle(tmpdir):
    vendored_joblib_home = os.path.dirname(joblib_vendored.__file__)
    sklearn_externals_home = os.path.dirname(externals.__file__)
    if not vendored_joblib_home.startswith(sklearn_externals_home):
        pytest.skip("joblib is physically unvendored (e.g. as in debian)")

    # Check that a pickle that references sklearn.external.joblib can load
    f = tmpdir.join('foo.pkl')
    f.write(b'\x80\x02csklearn.externals.joblib.numpy_pickle\nNumpyArrayWrappe'
            b'r\nq\x00)\x81q\x01}q\x02(U\x05dtypeq\x03cnumpy\ndtype\nq\x04U'
            b'\x02i8q\x05K\x00K\x01\x87q\x06Rq\x07(K\x03U\x01<q\x08NNNJ\xff'
            b'\xff\xff\xffJ\xff\xff\xff\xffK\x00tq\tbU\x05shapeq\nK\x01\x85q'
            b'\x0bU\x05orderq\x0cU\x01Cq\rU\x08subclassq\x0ecnumpy\nndarray\nq'
            b'\x0fU\nallow_mmapq\x10\x88ub\x01\x00\x00\x00\x00\x00\x00\x00.',
            mode='wb')

    joblib_vendored.load(str(f))


def test_site_joblib_dispatch():
    if os.environ.get('SKLEARN_SITE_JOBLIB', False):
        assert Parallel is joblib_site.Parallel
        assert delayed is joblib_site.delayed
        assert parallel_backend is joblib_site.parallel_backend
        assert Memory is joblib_site.Memory

        assert joblib_vendored.Parallel is not joblib_site.Parallel
        assert joblib_vendored.delayed is not joblib_site.delayed
        assert joblib_vendored.parallel_backend is not \
            joblib_site.parallel_backend
        assert joblib_vendored.Memory is not joblib_site.Memory

    else:
        assert Parallel is joblib_vendored.Parallel
        assert delayed is joblib_vendored.delayed
        assert parallel_backend is joblib_vendored.parallel_backend
        assert Memory is joblib_vendored.Memory
